package com.engine.organization.service.impl;

import com.api.browser.bean.SearchConditionItem;
import com.api.browser.bean.SearchConditionOption;
import com.api.browser.util.ConditionFactory;
import com.api.browser.util.ConditionType;
import com.cloudstore.eccom.result.WeaResultMsg;
import com.engine.core.impl.Service;
import com.engine.organization.component.OrganizationWeaTable;
import com.engine.organization.entity.commom.RecordInfo;
import com.engine.organization.entity.extend.ExtendInfoOperateType;
import com.engine.organization.entity.extend.param.ExtendInfoParams;
import com.engine.organization.entity.extend.po.ExtendInfoPO;
import com.engine.organization.entity.fieldset.param.FieldTransferParam;
import com.engine.organization.entity.jclimport.po.JclImportHistoryDetailPO;
import com.engine.organization.entity.jclimport.vo.JclImportHistoryDetailVO;
import com.engine.organization.entity.job.po.JobPO;
import com.engine.organization.entity.scheme.po.GradePO;
import com.engine.organization.entity.scheme.po.LevelPO;
import com.engine.organization.entity.scheme.po.SchemePO;
import com.engine.organization.enums.LogModuleNameEnum;
import com.engine.organization.enums.OperateTypeEnum;
import com.engine.organization.enums.OrgImportEnum;
import com.engine.organization.exception.OrganizationRunTimeException;
import com.engine.organization.mapper.comp.CompMapper;
import com.engine.organization.mapper.department.DepartmentMapper;
import com.engine.organization.mapper.employee.EmployeeMapper;
import com.engine.organization.mapper.extend.ExtMapper;
import com.engine.organization.mapper.extend.ExtendInfoMapper;
import com.engine.organization.mapper.hrmresource.SystemDataMapper;
import com.engine.organization.mapper.jclimport.JclImportHistoryDetailMapper;
import com.engine.organization.mapper.job.JobMapper;
import com.engine.organization.mapper.resource.ResourceMapper;
import com.engine.organization.mapper.scheme.GradeMapper;
import com.engine.organization.mapper.scheme.LevelMapper;
import com.engine.organization.mapper.scheme.SchemeMapper;
import com.engine.organization.service.ImportCommonService;
import com.engine.organization.thread.HrmResourceTriggerRunnable;
import com.engine.organization.thread.JobTriggerRunnable;
import com.engine.organization.thread.OrganizationSyncEc;
import com.engine.organization.util.HasRightUtil;
import com.engine.organization.util.OrganizationAssert;
import com.engine.organization.util.db.MapperProxyFactory;
import com.engine.organization.util.excel.ExcelUtil;
import com.engine.organization.util.saveimport.OrgImportUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import weaver.file.ImageFileManager;
import weaver.general.BaseBean;
import weaver.general.GCONST;
import weaver.general.Util;
import weaver.systeminfo.SystemEnv;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author:dxfeng
 * @createTime: 2022/06/29
 * @version: 1.0
 */
public class ImportCommonServiceImpl extends Service implements ImportCommonService {
    /**
     * extend_group
     */
    private static final Long COMPANY_TYPE = 1L;
    private static final Long DEPARTMENT_TYPE = 2L;
    private static final Long JOB_TYPE = 3L;
    private static final Long RESOURCE_TYPE = 4L;

    private static final String RIGHT_NAME = "DataImport:All";
    private static final String HRM_COMPANY = "hrmsubcompany";
    private static final String HRM_DEPARTMENT = "hrmdepartment";
    private static final String HRM_RESOURCE = "hrmresource";

    static Map<String, ExtendInfoPO> importFieldsMap;

    static {
        importFieldsMap = new HashMap<>();
        importFieldsMap.put("方案编号", ExtendInfoPO.builder().tableName("jcl_org_scheme").fieldName("scheme_no").fieldNameDesc("方案编号").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("方案名称", ExtendInfoPO.builder().tableName("jcl_org_scheme").fieldName("scheme_name").fieldNameDesc("方案名称").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职级编号", ExtendInfoPO.builder().tableName("jcl_org_grade").fieldName("grade_no").fieldNameDesc("职级编号").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职级名称", ExtendInfoPO.builder().tableName("jcl_org_grade").fieldName("grade_name").fieldNameDesc("职级名称").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职等编号", ExtendInfoPO.builder().tableName("jcl_org_level").fieldName("level_no").fieldNameDesc("职等编号").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职等名称", ExtendInfoPO.builder().tableName("jcl_org_level").fieldName("level_name").fieldNameDesc("职等名称").isrequired(1).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("方案说明", ExtendInfoPO.builder().tableName("jcl_org_scheme").fieldName("scheme_description").fieldNameDesc("方案说明").isrequired(0).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职等说明", ExtendInfoPO.builder().tableName("jcl_org_level").fieldName("level_description").fieldNameDesc("职等说明").isrequired(0).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());
        importFieldsMap.put("职级说明", ExtendInfoPO.builder().tableName("jcl_org_grade").fieldName("grade_description").fieldNameDesc("职级说明").isrequired(0).controlType(1).browserType("1").customValue("[\"input\",\"text\",\"100\"]").build());

    }

    private ExtendInfoMapper getExtendInfoMapper() {
        return MapperProxyFactory.getProxy(ExtendInfoMapper.class);
    }

    private SystemDataMapper getSystemDataMapper() {
        return MapperProxyFactory.getProxy(SystemDataMapper.class);
    }

    @Override
    public Map<String, Object> getImportFields(String importType) {
        String extendType = "";
        String tableName = "";
        String errorMsg = "";
        List<String> excludeFields = new ArrayList<>();
        //List<FieldTransferParam> includeFields = new ArrayList<>();
        //List<String> selectedList = new ArrayList<>();
        Map<String, Object> returnMaps = new HashMap<>();
        if (StringUtils.isNotBlank(importType)) {
            switch (importType) {
                case "company":
                    extendType = "1";
                    tableName = "JCL_ORG_COMP";
                    excludeFields.add("parent_company");
                    excludeFields.add("ec_company");
                    break;
                case "department":
                    extendType = "2";
                    tableName = "JCL_ORG_DEPT";
                    excludeFields.add("parent_dept");
                    excludeFields.add("ec_department");
                    break;
                case "jobtitle":
                    extendType = "3";
                    tableName = "JCL_ORG_JOB";
                    excludeFields.add("parent_job");
                    break;
                case "resource":
                    extendType = "4";
                    tableName = "JCL_ORG_HRMRESOURCE";
                    excludeFields.add("scheme_id");
                    excludeFields.add("sequence_id");
                    excludeFields.add("job_grade");
                    excludeFields.add("job_level");
                    break;
                case "joblevel":
                    extendType = "0";
                    tableName = "JCL_ORG_SCHEME";
                    break;
                default:
                    errorMsg = "未找到对应的导入类型，请确认";
                    break;
            }
        } else {
            errorMsg = "请选择导入类型";
        }
        OrganizationAssert.isBlank(errorMsg, errorMsg);
        if (StringUtils.isAnyEmpty(extendType, tableName)) {
            return returnMaps;
        }

        if ("joblevel".equals(importType)) {
            List<FieldTransferParam> fieldDatas = new ArrayList<>();
            fieldDatas.add(FieldTransferParam.builder().id("1").name("职级编号").build());
            fieldDatas.add(FieldTransferParam.builder().id("2").name("职级名称").build());
            fieldDatas.add(FieldTransferParam.builder().id("3").name("方案编号").build());
            fieldDatas.add(FieldTransferParam.builder().id("4").name("方案名称").build());
            fieldDatas.add(FieldTransferParam.builder().id("5").name("职等编号").build());
            fieldDatas.add(FieldTransferParam.builder().id("6").name("职等名称").build());
            fieldDatas.add(FieldTransferParam.builder().id("7").name("方案说明").build());
            fieldDatas.add(FieldTransferParam.builder().id("8").name("职级说明").build());
            fieldDatas.add(FieldTransferParam.builder().id("9").name("职等说明").build());
            List<String> selectedKeys = new ArrayList<>();
            for (int i = 0; i < fieldDatas.subList(0, 6).size(); i++) {
                selectedKeys.add(fieldDatas.get(i).getId());
            }
            returnMaps.put("data", fieldDatas);
            returnMaps.put("selectedKeys", selectedKeys);

        } else {
            List<ExtendInfoPO> infoPOList = getExtendInfoMapper().listFields(extendType, "", tableName, ExtendInfoOperateType.LIST.getValue(), "1");
            List<ExtendInfoPO> filterList = infoPOList.stream().filter(item -> !excludeFields.contains(item.getFieldName()) && 6 != item.getControlType()).collect(Collectors.toList());

            List<FieldTransferParam> fieldDatas = filterList.stream().map(item -> FieldTransferParam.builder().id(item.getId().toString()).name(item.getFieldNameDesc()).build()).collect(Collectors.toList());
            List<String> selectedKeys = filterList.stream().filter(item -> (0 == item.getIsSystemDefault()) && 1 == item.getIsrequired()).map(item -> item.getId().toString()).collect(Collectors.toList());
            returnMaps.put("data", fieldDatas);
            returnMaps.put("selectedKeys", selectedKeys);
        }

        return returnMaps;
    }

    @Override
    public Map<String, Object> getImportForm(String importType, String columns) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            if (StringUtils.isNotBlank(importType)) {
                String templatePath = getTemplatePath(importType, columns);
                switch (importType) {
                    case "company":
                        returnMap.put("condition", getCompanyImportForm(templatePath));
                        break;
                    case "department":
                        returnMap.put("condition", getDepartmentImportForm(templatePath));
                        break;
                    case "jobtitle":
                        returnMap.put("condition", getJobTitleImportForm(templatePath));
                        break;
                    case "resource":
                        returnMap.put("condition", getResourceImportForm(templatePath));
                        break;
                    case "joblevel":
                        returnMap.put("condition", getJobLevelImportForm(templatePath));
                        break;
                    default:
                        returnMap.put("status", "-1");
                        returnMap.put("message", "请选择合适的导入类型");
                        return returnMap;
                }
            }
            returnMap.put("status", "1");
        } catch (IOException e) {
            returnMap.put("status", "-1");
            returnMap.put("message", "导入模板生成异常");
            new BaseBean().writeLog(e);
        }
        return returnMap;
    }

    @Override
    public Map<String, Object> saveImport(Map<String, Object> params, HttpServletRequest request) {
        String importType = (String) params.get("importType");
        String operateType = (String) params.get("operateType");
        String excelFile = (String) params.get("excelfile");
        String keyField = (String) params.get("keyField");
        Map<String, Object> returnMap = new HashMap<>();
        if (StringUtils.isNotBlank(importType)) {
            switch (importType) {
                case "company":
                    returnMap.put("pId", companyImport(operateType, excelFile));
                    break;
                case "department":
                    returnMap.put("pId", departmentImport(operateType, excelFile));
                    break;
                case "jobtitle":
                    returnMap.put("pId", jobImport(operateType, excelFile));
                    break;
                case "resource":
                    returnMap.put("pId", hrmResourceImport(operateType, excelFile, keyField));
                    break;
                case "joblevel":
                    returnMap.put("pId", jobLevelImport(operateType, excelFile));
                    break;
                default:
                    break;
            }
        }

        // 列表
        // importstatus
        return returnMap;
    }

    @Override
    public Map<String, Object> getImportResult(Map<String, Object> params) {
        String pId = (String) params.get("pId");
        // 刷新引用状态
        OrganizationWeaTable<JclImportHistoryDetailVO> table = new OrganizationWeaTable<>(user, JclImportHistoryDetailVO.class);
        String sqlWhere = " where status = '0' and pId = '" + pId + "'";
        table.setSqlwhere(sqlWhere);
        WeaResultMsg result = new WeaResultMsg(false);
        result.putAll(table.makeDataResult());
        result.success();
        Map<String, Object> resultMap = new HashMap<>(result.getResultMap());
        resultMap.put("importStatus", "over");
        resultMap.put("succnum", MapperProxyFactory.getProxy(JclImportHistoryDetailMapper.class).countRows("1", pId));
        resultMap.put("failnum", MapperProxyFactory.getProxy(JclImportHistoryDetailMapper.class).countRows("0", pId));
        return resultMap;
    }

    @Override
    public Map<String, Object> getHasRight() {
        Map<String, Object> returnMap = new HashMap<>();
        boolean hasRight = HasRightUtil.hasRight(user, RIGHT_NAME, true);
        returnMap.put("hasRight", hasRight);
        return returnMap;
    }

    @Override
    public Map<String, Object> saveCommonImport(Map<String, Object> params) {
        String importModule = Util.null2String(params.get("importModule"));
        OrgImportEnum orgImport = OrgImportEnum.getOrgImportUtil(importModule);
        return orgImport.orgImport(params, user);
    }

    @Override
    public Map<String, Object> getCommonForm(String importModule) {
        Map<String, Object> returnMap = new HashMap<>();
        OrgImportEnum orgImport = OrgImportEnum.getOrgImportUtil(importModule);
        returnMap.put("condition", orgImport.orgForm(user));
        returnMap.put("status", "1");
        return returnMap;
    }


    /**
     * 分部导入表单
     */
    private List<Object> getCompanyImportForm(String templatePath) throws IOException {


        // 返回导入数据
        List<Object> lsGroup = new ArrayList<>();
        Map<String, Object> groupItem = new HashMap<>();
        List<Object> itemList = new ArrayList<>();

        groupItem.put("title", SystemEnv.getHtmlLabelName(1361, user.getLanguage()));
        groupItem.put("defaultshow", true);

        SearchConditionItem searchConditionItem;
        ConditionFactory conditionFactory = new ConditionFactory(user);
        //导入类型
        List<SearchConditionOption> statusOptions = new ArrayList<>();
        statusOptions.add(new SearchConditionOption("add", SystemEnv.getHtmlLabelName(611, user.getLanguage()), true));
        statusOptions.add(new SearchConditionOption("update", SystemEnv.getHtmlLabelName(17744, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24863, "importType", statusOptions);
        searchConditionItem.setValue("add");
        itemList.add(searchConditionItem);

        //模板文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.INPUT, 28576, "templet");
        searchConditionItem.setValue(templatePath);
        itemList.add(searchConditionItem);

        //Excel文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.RESOURCEIMG, 16630, "excelfile");
        itemList.add(searchConditionItem);

        groupItem.put("items", itemList);
        lsGroup.add(groupItem);


        itemList = new ArrayList<>();
        groupItem = new HashMap<>();
        groupItem.put("title", SystemEnv.getHtmlLabelName(33803, Util.getIntValue(user.getLanguage())));
        groupItem.put("defaultshow", true);
        List<Integer> lsPromptLabel = new ArrayList<>(); //提示信息
        lsPromptLabel.add(34275);
        lsPromptLabel.add(125452);
        lsPromptLabel.add(125466);
        //lsPromptLabel.add(522355);
        //lsPromptLabel.add(522356);
        //lsPromptLabel.add(522357);
        //lsPromptLabel.add(530411);

        for (int i = 0; i < lsPromptLabel.size(); i++) {
            Map<String, Object> item = new HashMap<>();
            item.put("index", (i + 1));
            String value = Util.toScreen(SystemEnv.getHtmlLabelName(lsPromptLabel.get(i), user.getLanguage()), user.getLanguage());
            if (i == 0) {
                value += SystemEnv.getHtmlLabelName(28576, user.getLanguage());
                item.put("link", templatePath);
            }
            item.put("value", value);
            itemList.add(item);
        }
        groupItem.put("items", itemList);
        lsGroup.add(groupItem);
        return lsGroup;
    }

    private List<Object> getDepartmentImportForm(String templatePath) throws IOException {

        // 返回导入数据
        List<Object> lsGroup = new ArrayList<>();
        Map<String, Object> groupItem = new HashMap<>();
        List<Object> itemList = new ArrayList<>();

        groupItem.put("title", SystemEnv.getHtmlLabelName(1361, user.getLanguage()));
        groupItem.put("defaultshow", true);

        SearchConditionItem searchConditionItem;
        ConditionFactory conditionFactory = new ConditionFactory(user);
        //导入类型
        List<SearchConditionOption> statusOptions = new ArrayList<>();
        statusOptions.add(new SearchConditionOption("add", SystemEnv.getHtmlLabelName(611, user.getLanguage()), true));
        statusOptions.add(new SearchConditionOption("update", SystemEnv.getHtmlLabelName(17744, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24863, "importType", statusOptions);
        searchConditionItem.setValue("add");
        itemList.add(searchConditionItem);

        //模板文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.INPUT, 28576, "templet");
        searchConditionItem.setValue(templatePath);
        itemList.add(searchConditionItem);

        //Excel文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.RESOURCEIMG, 16630, "excelfile");
        itemList.add(searchConditionItem);

        groupItem.put("items", itemList);
        lsGroup.add(groupItem);


        itemList = new ArrayList<>();
        groupItem = new HashMap<>();
        groupItem.put("title", SystemEnv.getHtmlLabelName(33803, Util.getIntValue(user.getLanguage())));
        groupItem.put("defaultshow", true);
        List<Integer> lsPromptLabel = new ArrayList<>(); //提示信息
        lsPromptLabel.add(34275);
        lsPromptLabel.add(125452);
        lsPromptLabel.add(125466);
        //lsPromptLabel.add(522355);
        //lsPromptLabel.add(522356);
        //lsPromptLabel.add(522357);
        //lsPromptLabel.add(530411);

        for (int i = 0; i < lsPromptLabel.size(); i++) {
            Map<String, Object> item = new HashMap<>();
            item.put("index", (i + 1));
            String value = Util.toScreen(SystemEnv.getHtmlLabelName(lsPromptLabel.get(i), user.getLanguage()), user.getLanguage());
            if (i == 0) {
                value += SystemEnv.getHtmlLabelName(28576, user.getLanguage());
                item.put("link", templatePath);
            }
            item.put("value", value);
            itemList.add(item);
        }
        groupItem.put("items", itemList);
        lsGroup.add(groupItem);
        return lsGroup;
    }

    private List<Object> getJobTitleImportForm(String templatePath) throws IOException {

        // 返回导入数据
        List<Object> lsGroup = new ArrayList<>();
        Map<String, Object> groupItem = new HashMap<>();
        List<Object> itemList = new ArrayList<>();

        groupItem.put("title", SystemEnv.getHtmlLabelName(1361, user.getLanguage()));
        groupItem.put("defaultshow", true);

        SearchConditionItem searchConditionItem;
        ConditionFactory conditionFactory = new ConditionFactory(user);

        //导入类型
        List<SearchConditionOption> statusOptions = new ArrayList<>();
        statusOptions.add(new SearchConditionOption("add", SystemEnv.getHtmlLabelName(611, user.getLanguage()), true));
        statusOptions.add(new SearchConditionOption("update", SystemEnv.getHtmlLabelName(17744, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24863, "importType", statusOptions);
        searchConditionItem.setValue("add");
        itemList.add(searchConditionItem);

        //模板文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.INPUT, 28576, "templet");
        searchConditionItem.setValue(templatePath);
        itemList.add(searchConditionItem);

        //Excel文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.RESOURCEIMG, 16630, "excelfile");
        itemList.add(searchConditionItem);

        groupItem.put("items", itemList);
        lsGroup.add(groupItem);


        itemList = new ArrayList<>();
        groupItem = new HashMap<>();
        groupItem.put("title", SystemEnv.getHtmlLabelName(33803, Util.getIntValue(user.getLanguage())));
        groupItem.put("defaultshow", true);
        List<Integer> lsPromptLabel = new ArrayList<>(); //提示信息
        lsPromptLabel.add(34275);
        lsPromptLabel.add(125452);

        for (int i = 0; i < lsPromptLabel.size(); i++) {
            Map<String, Object> item = new HashMap<>();
            item.put("index", (i + 1));
            String value = Util.toScreen(SystemEnv.getHtmlLabelName(lsPromptLabel.get(i), user.getLanguage()), user.getLanguage());
            if (i == 0) {
                value += SystemEnv.getHtmlLabelName(28576, user.getLanguage());
                item.put("link", templatePath);
            }
            item.put("value", value);
            itemList.add(item);
        }
        groupItem.put("items", itemList);
        lsGroup.add(groupItem);
        return lsGroup;
    }

    private List<Object> getJobLevelImportForm(String templatePath) throws IOException {

        // 返回导入数据
        List<Object> lsGroup = new ArrayList<>();
        Map<String, Object> groupItem = new HashMap<>();
        List<Object> itemList = new ArrayList<>();

        groupItem.put("title", SystemEnv.getHtmlLabelName(1361, user.getLanguage()));
        groupItem.put("defaultshow", true);

        SearchConditionItem searchConditionItem;
        ConditionFactory conditionFactory = new ConditionFactory(user);

        //导入类型
        List<SearchConditionOption> statusOptions = new ArrayList<>();
        statusOptions.add(new SearchConditionOption("add", SystemEnv.getHtmlLabelName(611, user.getLanguage()), true));
        statusOptions.add(new SearchConditionOption("update", SystemEnv.getHtmlLabelName(17744, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24863, "importType", statusOptions);
        searchConditionItem.setValue("add");
        itemList.add(searchConditionItem);

        //模板文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.INPUT, 28576, "templet");
        searchConditionItem.setValue(templatePath);
        itemList.add(searchConditionItem);

        //Excel文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.RESOURCEIMG, 16630, "excelfile");
        itemList.add(searchConditionItem);

        groupItem.put("items", itemList);
        lsGroup.add(groupItem);


        itemList = new ArrayList<>();
        groupItem = new HashMap<>();
        groupItem.put("title", SystemEnv.getHtmlLabelName(33803, Util.getIntValue(user.getLanguage())));
        groupItem.put("defaultshow", true);
        List<Integer> lsPromptLabel = new ArrayList<>(); //提示信息
        lsPromptLabel.add(34275);
        lsPromptLabel.add(125452);

        for (int i = 0; i < lsPromptLabel.size(); i++) {
            Map<String, Object> item = new HashMap<>();
            item.put("index", (i + 1));
            String value = Util.toScreen(SystemEnv.getHtmlLabelName(lsPromptLabel.get(i), user.getLanguage()), user.getLanguage());
            if (i == 0) {
                value += SystemEnv.getHtmlLabelName(28576, user.getLanguage());
                item.put("link", templatePath);
            }
            item.put("value", value);
            itemList.add(item);
        }
        Map<String, Object> item = new HashMap<>();
        item.put("index", 3);
        item.put("value", "多个职等之间使用英文状态下逗号”,“隔开");
        itemList.add(item);
        groupItem.put("items", itemList);
        lsGroup.add(groupItem);
        return lsGroup;
    }

    private List<Object> getResourceImportForm(String templatePath) throws IOException {

        // 返回导入数据
        List<Object> lsGroup = new ArrayList<>();
        Map<String, Object> groupItem = new HashMap<>();
        List<Object> itemList = new ArrayList<>();

        groupItem.put("title", SystemEnv.getHtmlLabelName(1361, user.getLanguage()));
        groupItem.put("defaultshow", true);

        SearchConditionItem searchConditionItem;
        ConditionFactory conditionFactory = new ConditionFactory(user);

        //重复验证字段
        List<SearchConditionOption> statusOptions = new ArrayList<>();
        statusOptions.add(new SearchConditionOption("last_name", SystemEnv.getHtmlLabelName(413, user.getLanguage())));
        statusOptions.add(new SearchConditionOption("work_code", SystemEnv.getHtmlLabelName(714, user.getLanguage()), true));
        //statusOptions.add(new SearchConditionOption("loginid", SystemEnv.getHtmlLabelName(412, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24638, "keyField", statusOptions);
        searchConditionItem.setValue("last_name");
        itemList.add(searchConditionItem);

        //导入类型
        List<SearchConditionOption> statusOptions1 = new ArrayList<>();
        statusOptions1.add(new SearchConditionOption("add", SystemEnv.getHtmlLabelName(611, user.getLanguage()), true));
        statusOptions1.add(new SearchConditionOption("update", SystemEnv.getHtmlLabelName(17744, user.getLanguage())));
        searchConditionItem = conditionFactory.createCondition(ConditionType.SELECT, 24863, "importType", statusOptions1);
        searchConditionItem.setValue("add");
        itemList.add(searchConditionItem);

        //模板文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.INPUT, 28576, "templet");
        searchConditionItem.setValue(templatePath);
        itemList.add(searchConditionItem);

        //Excel文件
        searchConditionItem = conditionFactory.createCondition(ConditionType.RESOURCEIMG, 16630, "excelfile");
        itemList.add(searchConditionItem);

        groupItem.put("items", itemList);
        lsGroup.add(groupItem);


        itemList = new ArrayList<>();
        groupItem = new HashMap<>();
        groupItem.put("title", SystemEnv.getHtmlLabelName(33803, Util.getIntValue(user.getLanguage())));
        groupItem.put("defaultshow", true);
        List<Integer> lsPromptLabel = new ArrayList<>(); //提示信息
        lsPromptLabel.add(128520);
        lsPromptLabel.add(125868);
        lsPromptLabel.add(128365);
        //lsPromptLabel.add(81697);
        //lsPromptLabel.add(81698);
        lsPromptLabel.add(81699);
        lsPromptLabel.add(516263);
        //lsPromptLabel.add(81701);
        //lsPromptLabel.add(388880);
        lsPromptLabel.add(81702);
        lsPromptLabel.add(81703);
        lsPromptLabel.add(125869);

        for (int i = 0; i < lsPromptLabel.size(); i++) {
            Map<String, Object> item = new HashMap<>();
            item.put("index", (i + 1));
            String value = Util.toScreen(SystemEnv.getHtmlLabelName(lsPromptLabel.get(i), user.getLanguage()), user.getLanguage());
            if (i == 0) {
                value += SystemEnv.getHtmlLabelName(28576, user.getLanguage());
                item.put("link", templatePath);
            }
            item.put("value", value);
            itemList.add(item);
        }
        groupItem.put("items", itemList);
        lsGroup.add(groupItem);
        return lsGroup;
    }


    /**
     * 导入分部
     */
    private Long companyImport(String operateType, String excelFile) {
        Long importHistoryId = OrgImportUtil.saveImportLog("company", operateType, user);
        JclImportHistoryDetailPO historyDetailPO;

        ImageFileManager manager = new ImageFileManager();
        manager.getImageFileInfoById(Util.getIntValue(excelFile));
        XSSFWorkbook workbook;
        try {
            workbook = new XSSFWorkbook(manager.getInputStream());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        // 当前sheet
        XSSFSheet sheetAt = workbook.getSheetAt(0);
        int lastRow = sheetAt.getLastRowNum();
        List<ExtendInfoPO> extendInfoPOS = new ArrayList<>();
        OrganizationAssert.isTrue(lastRow > 0, "导入数据为空");
        short lastCellNum = sheetAt.getRow(0).getLastCellNum();


        // 遍历每一行数据
        nextRow:
        for (int i = 0; i <= lastRow; i++) {
            historyDetailPO = new JclImportHistoryDetailPO();
            historyDetailPO.setPid(importHistoryId);
            XSSFRow row = sheetAt.getRow(i);
            if (null == row) {
                continue;
            }
            // 组装待处理数据
            Map<String, Object> map = new HashMap<>();
            Integer parentCompanyId = null;
            String companyName = "";

            historyDetailPO.setRowNums(String.valueOf(i + 1));
            for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
                XSSFCell cell = row.getCell((short) cellIndex);
                String cellValue = getCellValue(cell).trim();
                if (i == 0) {
                    List<ExtendInfoPO> infoPOList = getExtendInfoMapper().getInfoByExtendAndLabelName(COMPANY_TYPE, cellValue);
                    boolean isBreak = isBreak(historyDetailPO, cellValue, infoPOList);
                    if (isBreak) {
                        break nextRow;
                    }
                    // 首行 初始化字段信息
                    ExtendInfoPO extendInfoPO = infoPOList.get(0);
                    extendInfoPOS.add(extendInfoPO);
                } else {
                    ExtendInfoPO infoPO = extendInfoPOS.get(cellIndex);
                    // 数据校验
                    if (infoPO.getIsrequired() == 1 && StringUtils.isBlank(cellValue) && !"comp_no".equalsIgnoreCase(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "为必填项");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }

                    Object reallyValue;
                    try {
                        reallyValue = OrgImportUtil.getReallyValue(infoPO, cellValue);
                    } catch (Exception e) {
                        historyDetailPO.setOperateDetail(cellValue + "转换失败");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    if (StringUtils.isNotBlank(cellValue) && StringUtils.isBlank(Util.null2String(reallyValue))) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "数据转换失败,未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    map.put(infoPO.getFieldName(), reallyValue);
                    if ("comp_name".equals(infoPO.getFieldName())) {
                        historyDetailPO.setRelatedName(cellValue);
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("分部层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (int index = 0; index < split.length - 1; index++) {
                                parentCompanyId = MapperProxyFactory.getProxy(CompMapper.class).getIdByNameAndPid(split[index], parentCompanyId == null ? 0 : parentCompanyId);
                                if (null == parentCompanyId) {
                                    historyDetailPO.setOperateDetail(split[index] + "分部未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                        map.put("parent_company", parentCompanyId);
                        map.put("ec_company", parentCompanyId);

                        companyName = split[split.length - 1];
                        map.put("comp_name", companyName);
                    }

                    if ("comp_principal".equals(infoPO.getFieldName())) {
                        boolean hasSameName = hasSameName(historyDetailPO, cellValue);
                        if (hasSameName) {
                            continue nextRow;
                        }
                    }
                }
            }
            // 校验、数据交互
            if (i == 0) {
                continue;
            }
            String compNo = (String) map.get("comp_no");
            Integer companyId = MapperProxyFactory.getProxy(CompMapper.class).getIdByNameAndPid(companyName, parentCompanyId == null ? 0 : parentCompanyId);
            if ("add".equals(operateType)) {
                if (companyId != null) {
                    historyDetailPO.setOperateDetail("数据已存在");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                // 自动编号
                try {
                    compNo = CompServiceImpl.repeatDetermine(compNo);
                } catch (OrganizationRunTimeException e) {
                    historyDetailPO.setOperateDetail(e.getMessage());
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                map.put("comp_no", compNo);
                map.put("creator", user.getUID());
                map.put("delete_type", 0);
                map.put("create_time", new Date());
                // 禁用表示为空，默认启用
                if (StringUtils.isBlank(Util.null2String(map.get("forbidden_tag")))) {
                    map.put("forbidden_tag", 0);
                }
                String showOrder = Util.null2String(map.get("show_order"));
                if (StringUtils.isBlank(showOrder)) {
                    Integer maxShowOrder = MapperProxyFactory.getProxy(CompMapper.class).getMaxShowOrder();
                    if (null == maxShowOrder) {
                        maxShowOrder = 0;
                    }
                    map.put("show_order", maxShowOrder + 1);
                }

                Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.COMPANY, OperateTypeEnum.ADD, map, false).sync();
                if (isThrowError(syncMap)) {
                    // 查询UUID
                    String ecCompanyID = Util.null2String(syncMap.get("id"));
                    RecordInfo recordInfo = getSystemDataMapper().getHrmObjectByID(HRM_COMPANY, ecCompanyID);
                    map.put("uuid", recordInfo.getUuid());
                    ExtendInfoParams infoParams = ExtendInfoParams.builder().tableName("JCL_ORG_COMP").params(map).build();
                    MapperProxyFactory.getProxy(ExtMapper.class).insertTable(infoParams);
                    // 刷新组织架构图
                    //TODO new CompanyTriggerRunnable(infoParams.getId()).run();
                    historyDetailPO.setOperateDetail("添加成功");
                    historyDetailPO.setStatus("1");
                } else {
                    historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                    historyDetailPO.setStatus("0");
                }
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
            } else if ("update".equals(operateType)) {
                // 查询对应ID
                if (StringUtils.isNotBlank(compNo)) {
                    if (companyId == null) {
                        historyDetailPO.setOperateDetail("未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue;
                    }
                    if (checkRepeatNo(compNo, COMPANY_TYPE, Long.valueOf(companyId))) {
                        map.put("update_time", new Date());
                        map.put("id", companyId);
                        Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.COMPANY, OperateTypeEnum.UPDATE, map, false).sync();
                        if (isThrowError(syncMap)) {
                            map.remove("id");
                            MapperProxyFactory.getProxy(ExtMapper.class).updateTable(ExtendInfoParams.builder().id(Long.valueOf(companyId)).tableName("JCL_ORG_COMP").params(map).build());
                            // 刷新组织架构图
                            //TODO new CompanyTriggerRunnable(companyId).run();
                            historyDetailPO.setOperateDetail("更新成功");
                            historyDetailPO.setStatus("1");
                        } else {
                            historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                            historyDetailPO.setStatus("0");
                        }
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    } else {
                        historyDetailPO.setOperateDetail(compNo + "编号已存在");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    }
                } else {
                    historyDetailPO.setOperateDetail("编号为空,更新失败");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                }
            }
        }
        return importHistoryId;
    }

    /**
     * 导入部门
     */
    private Long departmentImport(String operateType, String excelFile) {
        Long importHistoryId = OrgImportUtil.saveImportLog("department", operateType, user);
        JclImportHistoryDetailPO historyDetailPO;

        ImageFileManager manager = new ImageFileManager();
        manager.getImageFileInfoById(Util.getIntValue(excelFile));
        XSSFWorkbook workbook;
        try {
            workbook = new XSSFWorkbook(manager.getInputStream());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        // 当前sheet
        XSSFSheet sheetAt = workbook.getSheetAt(0);
        int lastRow = sheetAt.getLastRowNum();
        List<ExtendInfoPO> extendInfoPOS = new ArrayList<>();
        OrganizationAssert.isTrue(lastRow > 0, "导入数据为空");
        short lastCellNum = sheetAt.getRow(0).getLastCellNum();


        // 遍历每一行数据
        nextRow:
        for (int i = 0; i <= lastRow; i++) {
            historyDetailPO = new JclImportHistoryDetailPO();
            historyDetailPO.setPid(importHistoryId);
            XSSFRow row = sheetAt.getRow(i);
            if (null == row) {
                continue;
            }
            // 组装待处理数据
            Map<String, Object> map = new HashMap<>();
            Integer parentCompanyId = null;
            Integer parentDepartmentId = null;
            String departmentName = "";

            historyDetailPO.setRowNums(String.valueOf(i + 1));
            for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
                XSSFCell cell = row.getCell((short) cellIndex);
                String cellValue = getCellValue(cell).trim();
                if (i == 0) {
                    // 首行 初始化字段信息
                    List<ExtendInfoPO> infoPOList = getExtendInfoMapper().getInfoByExtendAndLabelName(DEPARTMENT_TYPE, cellValue);
                    boolean isBreak = isBreak(historyDetailPO, cellValue, infoPOList);
                    if (isBreak) {
                        break nextRow;
                    }
                    ExtendInfoPO extendInfoPO = infoPOList.get(0);
                    extendInfoPOS.add(extendInfoPO);
                } else {
                    ExtendInfoPO infoPO = extendInfoPOS.get(cellIndex);
                    // 数据校验
                    if (infoPO.getIsrequired() == 1 && StringUtils.isBlank(cellValue) && !"dept_no".equalsIgnoreCase(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "为必填项");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }

                    Object reallyValue;
                    try {
                        reallyValue = OrgImportUtil.getReallyValue(infoPO, cellValue);
                    } catch (Exception e) {
                        historyDetailPO.setOperateDetail(cellValue + "转换失败");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    if (StringUtils.isNotBlank(cellValue) && StringUtils.isBlank(Util.null2String(reallyValue)) && !"ec_company".equals(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "数据转换失败,未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    map.put(infoPO.getFieldName(), reallyValue);
                    // 上级分部
                    if ("ec_company".equals(infoPO.getFieldName())) {
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("分部层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentCompanyId = MapperProxyFactory.getProxy(CompMapper.class).getIdByNameAndPid(s, parentCompanyId == null ? 0 : parentCompanyId);
                                if (null == parentCompanyId) {
                                    historyDetailPO.setOperateDetail(cellValue + "分部未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                    }

                    if ("dept_name".equals(infoPO.getFieldName())) {
                        if (null == parentCompanyId) {
                            historyDetailPO.setOperateDetail(cellValue + "所属分部未找到");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }
                        historyDetailPO.setRelatedName(cellValue);
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("部门层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (int index = 0; index < split.length - 1; index++) {
                                parentDepartmentId = MapperProxyFactory.getProxy(DepartmentMapper.class).getIdByNameAndPid(split[index], parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId);
                                if (null == parentDepartmentId) {
                                    historyDetailPO.setOperateDetail(split[index] + "部门未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                        map.put("parent_comp", parentCompanyId);
                        map.put("ec_company", parentCompanyId);
                        map.put("parent_dept", parentDepartmentId);
                        map.put("ec_department", parentDepartmentId);

                        departmentName = split[split.length - 1];
                        map.put("dept_name", departmentName);
                    }
                    if ("dept_principal".equals(infoPO.getFieldName())) {
                        boolean hasSameName = hasSameName(historyDetailPO, cellValue);
                        if (hasSameName) {
                            continue nextRow;
                        }
                    }
                }
            }
            // 校验、数据交互
            if (i == 0) {
                continue;
            }
            String deptNo = (String) map.get("dept_no");
            Integer departmentId = MapperProxyFactory.getProxy(DepartmentMapper.class).getIdByNameAndPid(departmentName, parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId);
            if ("add".equals(operateType)) {
                if (departmentId != null) {
                    historyDetailPO.setOperateDetail("数据已存在");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                // 自动编号
                try {
                    deptNo = DepartmentServiceImpl.repeatDetermine(deptNo);
                } catch (OrganizationRunTimeException e) {
                    historyDetailPO.setOperateDetail(e.getMessage());
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                map.put("dept_no", deptNo);
                map.put("creator", user.getUID());
                map.put("delete_type", 0);
                map.put("create_time", new Date());
                // 禁用表示为空，默认启用
                if (StringUtils.isBlank(Util.null2String(map.get("forbidden_tag")))) {
                    map.put("forbidden_tag", 0);
                }
                String showOrder = Util.null2String(map.get("show_order"));
                if (StringUtils.isBlank(showOrder)) {
                    Double maxShowOrder = MapperProxyFactory.getProxy(DepartmentMapper.class).getMaxShowOrder();
                    if (null == maxShowOrder) {
                        maxShowOrder = 0D;
                    }
                    map.put("show_order", maxShowOrder + 1);
                }
                Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.DEPARTMENT, OperateTypeEnum.ADD, map, false).sync();
                if (isThrowError(syncMap)) {
                    // 查询UUID
                    String ecDepartmentID = Util.null2String(syncMap.get("id"));
                    RecordInfo recordInfo = getSystemDataMapper().getHrmObjectByID(HRM_DEPARTMENT, ecDepartmentID);
                    map.put("uuid", recordInfo.getUuid());
                    ExtendInfoParams infoParams = ExtendInfoParams.builder().tableName("JCL_ORG_DEPT").params(map).build();
                    MapperProxyFactory.getProxy(ExtMapper.class).insertTable(infoParams);
                    // 刷新组织架构图
                    //TODO new DepartmentTriggerRunnable(infoParams.getId()).run();
                    map.put("id", infoParams.getId());
                    historyDetailPO.setOperateDetail("添加成功");
                    historyDetailPO.setStatus("1");
                } else {
                    historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                    historyDetailPO.setStatus("0");
                }
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
            } else if ("update".equals(operateType)) {
                // 查询对应ID
                if (StringUtils.isNotBlank(deptNo)) {
                    if (departmentId == null) {
                        historyDetailPO.setOperateDetail("未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue;
                    }
                    if (checkRepeatNo(deptNo, DEPARTMENT_TYPE, Long.valueOf(departmentId))) {
                        map.put("update_time", new Date());
                        map.put("id", departmentId);
                        Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.DEPARTMENT, OperateTypeEnum.UPDATE, map, false).sync();
                        if (isThrowError(syncMap)) {
                            map.remove("id");
                            MapperProxyFactory.getProxy(ExtMapper.class).updateTable(ExtendInfoParams.builder().id(Long.valueOf(departmentId)).tableName("JCL_ORG_DEPT").params(map).build());
                            // 刷新组织架构图
                            //TODO new DepartmentTriggerRunnable(departmentId).run();
                            historyDetailPO.setOperateDetail("更新成功");
                            historyDetailPO.setStatus("1");
                        } else {
                            historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                            historyDetailPO.setStatus("0");
                        }
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    } else {
                        historyDetailPO.setOperateDetail(deptNo + "编号已存在");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    }
                } else {
                    historyDetailPO.setOperateDetail("编号为空,更新失败");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                }
            }
        }
        return importHistoryId;
    }

    /**
     * 岗位导入
     */
    private Long jobImport(String operateType, String excelFile) {
        Long importHistoryId = OrgImportUtil.saveImportLog("jobtitle", operateType, user);
        JclImportHistoryDetailPO historyDetailPO;

        ImageFileManager manager = new ImageFileManager();
        manager.getImageFileInfoById(Util.getIntValue(excelFile));
        XSSFWorkbook workbook;
        try {
            workbook = new XSSFWorkbook(manager.getInputStream());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        // 当前sheet
        XSSFSheet sheetAt = workbook.getSheetAt(0);
        int lastRow = sheetAt.getLastRowNum();
        List<ExtendInfoPO> extendInfoPOS = new ArrayList<>();
        OrganizationAssert.isTrue(lastRow > 0, "导入数据为空");
        short lastCellNum = sheetAt.getRow(0).getLastCellNum();

        // 遍历每一行数据
        nextRow:
        for (int i = 0; i <= lastRow; i++) {
            historyDetailPO = new JclImportHistoryDetailPO();
            historyDetailPO.setPid(importHistoryId);
            XSSFRow row = sheetAt.getRow(i);
            if (null == row) {
                continue;
            }
            // 组装待处理数据
            Map<String, Object> map = new HashMap<>();
            Integer parentCompanyId = null;
            Integer parentDepartmentId = null;
            Long parentJobId = null;
            String jobName = "";

            historyDetailPO.setRowNums(String.valueOf(i + 1));
            for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
                XSSFCell cell = row.getCell((short) cellIndex);
                String cellValue = getCellValue(cell).trim();
                if (i == 0) {
                    // 首行 初始化字段信息
                    List<ExtendInfoPO> infoPOList = getExtendInfoMapper().getInfoByExtendAndLabelName(JOB_TYPE, cellValue);
                    boolean isBreak = isBreak(historyDetailPO, cellValue, infoPOList);
                    if (isBreak) {
                        break nextRow;
                    }
                    ExtendInfoPO extendInfoPO = infoPOList.get(0);
                    extendInfoPOS.add(extendInfoPO);
                } else {
                    ExtendInfoPO infoPO = extendInfoPOS.get(cellIndex);
                    // 数据校验
                    if (infoPO.getIsrequired() == 1 && StringUtils.isBlank(cellValue) && !"job_no".equalsIgnoreCase(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "为必填项");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }

                    Object reallyValue;
                    try {
                        reallyValue = OrgImportUtil.getReallyValue(infoPO, cellValue);
                    } catch (Exception e) {
                        historyDetailPO.setOperateDetail(cellValue + "转换失败");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    if (StringUtils.isNotBlank(cellValue) && StringUtils.isBlank(Util.null2String(reallyValue)) && !"ec_company".equals(infoPO.getFieldName()) && !"ec_department".equals(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "数据转换失败,未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    map.put(infoPO.getFieldName(), reallyValue);
                    // 上级分部
                    if ("ec_company".equals(infoPO.getFieldName())) {
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("分部层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentCompanyId = MapperProxyFactory.getProxy(CompMapper.class).getIdByNameAndPid(s, parentCompanyId == null ? 0 : parentCompanyId);
                                if (null == parentCompanyId) {
                                    historyDetailPO.setOperateDetail(cellValue + "分部未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                    }


                    if ("ec_department".equals(infoPO.getFieldName())) {
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("部门层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentDepartmentId = MapperProxyFactory.getProxy(DepartmentMapper.class).getIdByNameAndPid(s, parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId);
                                if (null == parentDepartmentId) {
                                    historyDetailPO.setOperateDetail(cellValue + "部门未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                    }

                    if ("job_name".equals(infoPO.getFieldName())) {
                        if (null == parentCompanyId) {
                            historyDetailPO.setOperateDetail(cellValue + "所属分部未找到");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }
                        historyDetailPO.setRelatedName(cellValue);
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("岗位层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (int index = 0; index < split.length - 1; index++) {
                                parentJobId = MapperProxyFactory.getProxy(JobMapper.class).getIdByNameAndPid(split[index], parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId, parentJobId == null ? 0 : parentJobId);
                                if (null == parentJobId) {
                                    historyDetailPO.setOperateDetail(split[index] + "岗位未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                        map.put("parent_comp", parentCompanyId);
                        map.put("parent_dept", parentDepartmentId);
                        map.put("ec_company", parentCompanyId);
                        map.put("ec_department", parentDepartmentId);
                        map.put("parent_job", parentJobId);
                        jobName = split[split.length - 1];
                        map.put("job_name", jobName);
                        map.put("is_key", StringUtils.isBlank(Util.null2String(map.get("is_key"))) ? 0 : map.get("is_key"));
                    }
                }
            }
            // 校验、数据交互
            if (i == 0) {
                continue;
            }
            String jobNo = (String) map.get("job_no");
            Long jobId = MapperProxyFactory.getProxy(JobMapper.class).getIdByNameAndPid(jobName, parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId, parentJobId == null ? 0 : parentJobId);
            if ("add".equals(operateType)) {
                if (jobId != null) {
                    historyDetailPO.setOperateDetail("数据已存在");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                // 自动编号
                try {
                    jobNo = JobServiceImpl.repeatDetermine(jobNo);
                } catch (OrganizationRunTimeException e) {
                    historyDetailPO.setOperateDetail(e.getMessage());
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                map.put("job_no", jobNo);
                map.put("creator", user.getUID());
                map.put("delete_type", 0);
                map.put("create_time", new Date());
                // 禁用表示为空，默认启用
                if (StringUtils.isBlank(Util.null2String(map.get("forbidden_tag")))) {
                    map.put("forbidden_tag", 0);
                }
                String showOrder = Util.null2String(map.get("show_order"));
                if (StringUtils.isBlank(showOrder)) {
                    Integer maxShowOrder = MapperProxyFactory.getProxy(JobMapper.class).getMaxShowOrder();
                    if (null == maxShowOrder) {
                        maxShowOrder = 0;
                    }
                    map.put("show_order", maxShowOrder + 1);
                }
                ExtendInfoParams infoParams = ExtendInfoParams.builder().tableName("JCL_ORG_JOB").params(map).build();

                map.put("jobactivityid", JobServiceImpl.JOB_ACTIVITY_ID);
                Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.JOB, OperateTypeEnum.ADD, map, false).sync();
                if (isThrowError(syncMap)) {
                    boolean assertNameRepeat = JobServiceImpl.assertNameRepeat(null, Util.null2String(map.get("ec_department")), Util.null2String(map.get("parent_job")), Util.null2String(map.get("job_name")));
                    if (assertNameRepeat) {
                        map.remove("jobactivityid");
                        MapperProxyFactory.getProxy(ExtMapper.class).insertTable(infoParams);
                        // 更新组织架构图
                        new JobTriggerRunnable(infoParams.getId()).run();
                        historyDetailPO.setOperateDetail("添加成功");
                        historyDetailPO.setStatus("1");
                    } else {
                        historyDetailPO.setOperateDetail("岗位名称已存在");
                        historyDetailPO.setStatus("0");
                    }
                } else {
                    historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                    historyDetailPO.setStatus("0");
                }
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
            } else if ("update".equals(operateType)) {
                // 查询对应ID
                if (StringUtils.isNotBlank(jobNo)) {
                    if (jobId == null) {
                        historyDetailPO.setOperateDetail("未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue;
                    }
                    if (checkRepeatNo(jobNo, JOB_TYPE, jobId)) {
                        map.put("update_time", new Date());
                        map.put("jobactivityid", JobServiceImpl.JOB_ACTIVITY_ID);
                        map.put("id", jobId);
                        Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.JOB, OperateTypeEnum.UPDATE, map, MapperProxyFactory.getProxy(JobMapper.class).getJobById(jobId), false).sync();
                        if (isThrowError(syncMap)) {
                            boolean assertNameRepeat = JobServiceImpl.assertNameRepeat(jobId.toString(), Util.null2String(map.get("ec_department")), Util.null2String(map.get("parent_job")), Util.null2String(map.get("job_name")));
                            if (assertNameRepeat) {
                                map.remove("id");
                                map.remove("jobactivityid");
                                MapperProxyFactory.getProxy(ExtMapper.class).updateTable(ExtendInfoParams.builder().id(jobId).tableName("JCL_ORG_JOB").params(map).build());
                                // 更新组织架构图
                                new JobTriggerRunnable(jobId).run();
                                historyDetailPO.setOperateDetail("更新成功");
                                historyDetailPO.setStatus("1");
                            } else {
                                historyDetailPO.setOperateDetail("岗位名称已存在");
                                historyDetailPO.setStatus("0");
                            }
                        } else {
                            historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                            historyDetailPO.setStatus("0");
                        }
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    } else {
                        historyDetailPO.setOperateDetail(jobNo + "编号已存在");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    }
                } else {
                    historyDetailPO.setOperateDetail("编号为空,更新失败");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                }
            }
        }
        return importHistoryId;
    }

    private Long hrmResourceImport(String operateType, String excelFile, String keyField) {
        Long importHistoryId = OrgImportUtil.saveImportLog("resource", operateType, user);
        JclImportHistoryDetailPO historyDetailPO;

        ImageFileManager manager = new ImageFileManager();
        manager.getImageFileInfoById(Util.getIntValue(excelFile));
        XSSFWorkbook workbook;
        try {
            workbook = new XSSFWorkbook(manager.getInputStream());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        // 当前sheet
        XSSFSheet sheetAt = workbook.getSheetAt(0);
        int lastRow = sheetAt.getLastRowNum();
        List<ExtendInfoPO> extendInfoPOS = new ArrayList<>();
        OrganizationAssert.isTrue(lastRow > 0, "导入数据为空");
        short lastCellNum = sheetAt.getRow(0).getLastCellNum();

        // 遍历每一行数据
        nextRow:
        for (int i = 0; i <= lastRow; i++) {
            historyDetailPO = new JclImportHistoryDetailPO();
            historyDetailPO.setPid(importHistoryId);
            XSSFRow row = sheetAt.getRow(i);
            if (null == row) {
                continue;
            }
            // 组装待处理数据
            Map<String, Object> map = new HashMap<>();
            Integer parentCompanyId = null;
            Integer parentDepartmentId = null;
            Long parentJobId = null;

            historyDetailPO.setRowNums(String.valueOf(i + 1));
            for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
                XSSFCell cell = row.getCell((short) cellIndex);
                String cellValue = getCellValue(cell).trim();
                if (i == 0) {
                    // 首行 初始化字段信息
                    List<ExtendInfoPO> infoPOList = getExtendInfoMapper().getInfoByExtendAndLabelName(RESOURCE_TYPE, cellValue);
                    boolean isBreak = isBreak(historyDetailPO, cellValue, infoPOList);
                    if (isBreak) {
                        break nextRow;
                    }
                    ExtendInfoPO extendInfoPO = infoPOList.get(0);
                    extendInfoPOS.add(extendInfoPO);
                } else {
                    ExtendInfoPO infoPO = extendInfoPOS.get(cellIndex);
                    // 数据校验
                    if (infoPO.getIsrequired() == 1 && StringUtils.isBlank(cellValue) && !"work_code".equalsIgnoreCase(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "为必填项");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }

                    Object reallyValue;
                    try {
                        reallyValue = OrgImportUtil.getReallyValue(infoPO, cellValue);
                    } catch (Exception e) {
                        historyDetailPO.setOperateDetail(cellValue + "转换失败");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    if (StringUtils.isNotBlank(cellValue) && StringUtils.isBlank(Util.null2String(reallyValue)) && !"ec_company".equals(infoPO.getFieldName()) && !"ec_department".equals(infoPO.getFieldName()) && !"job_title".equals(infoPO.getFieldName())) {
                        historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "数据转换失败,未找到对应数据");
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue nextRow;
                    }
                    map.put(infoPO.getFieldName(), reallyValue);
                    // 上级分部
                    if ("ec_company".equals(infoPO.getFieldName())) {
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("分部层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentCompanyId = MapperProxyFactory.getProxy(CompMapper.class).getIdByNameAndPid(s, parentCompanyId == null ? 0 : parentCompanyId);
                                if (null == parentCompanyId) {
                                    historyDetailPO.setOperateDetail(cellValue + "分部未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                    }


                    if ("ec_department".equals(infoPO.getFieldName())) {
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("部门层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentDepartmentId = MapperProxyFactory.getProxy(DepartmentMapper.class).getIdByNameAndPid(s, parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId);
                                if (null == parentDepartmentId) {
                                    historyDetailPO.setOperateDetail(cellValue + "部门未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                    }

                    if ("job_title".equals(infoPO.getFieldName())) {
                        if (null == parentCompanyId) {
                            historyDetailPO.setOperateDetail(cellValue + "所属分部未找到");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }
                        String[] split = cellValue.split(">");
                        if (split.length > 0) {
                            if (split.length > 8) {
                                historyDetailPO.setOperateDetail("岗位层级不能大于10");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                            for (String s : split) {
                                parentJobId = MapperProxyFactory.getProxy(JobMapper.class).getIdByNameAndPid(s, parentCompanyId, parentDepartmentId == null ? 0 : parentDepartmentId, parentJobId == null ? 0 : parentJobId);
                                if (null == parentJobId) {
                                    historyDetailPO.setOperateDetail(s + "岗位未找到对应数据");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue nextRow;
                                }
                            }
                        }
                        map.put("company_id", parentCompanyId);
                        map.put("department_id", parentDepartmentId);
                        map.put("ec_company", parentCompanyId);
                        map.put("ec_department", parentDepartmentId);

                        map.put("job_title", parentJobId);
                    }
                }
            }
            // 校验、数据交互
            if (i == 0) {
                continue;
            }
            // 根据岗位获取等级方案、岗位序列、职等、职级
            if (null != parentJobId) {
                JobPO jobById = MapperProxyFactory.getProxy(JobMapper.class).getJobById(parentJobId);
                map.put("scheme_id", jobById.getSchemeId());
                map.put("sequence_id", jobById.getSequenceId());
                map.put("job_level", jobById.getLevelId());
                map.put("job_grade", jobById.getGradeId());
            }

            String keyFieldValue = Util.null2String(map.get(keyField));
            if (StringUtils.isBlank(keyFieldValue)) {
                historyDetailPO.setOperateDetail("所选重复验证字段为空");
                historyDetailPO.setStatus("0");
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                continue;
            }
            historyDetailPO.setRelatedName(keyFieldValue);
            //  判断当前人员是否存在
            boolean hasSameKeyFieldValue = hasSameKeyFieldValue(historyDetailPO, keyField, keyFieldValue);
            if (hasSameKeyFieldValue) {
                continue;
            }

            Long resourceId = MapperProxyFactory.getProxy(ResourceMapper.class).getIdByKeyField(keyField, keyFieldValue).get(0);
            if ("add".equals(operateType)) {
                if (resourceId != null) {
                    historyDetailPO.setOperateDetail(keyFieldValue + "已存在");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }

                // 自动编号
                String workCode = Util.null2String(map.get("work_code"));
                try {
                    // 自动编号
                    try {
                        workCode = HrmResourceServiceImpl.repeatDetermine(workCode);
                    } catch (OrganizationRunTimeException e) {
                        historyDetailPO.setOperateDetail(e.getMessage());
                        historyDetailPO.setStatus("0");
                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                        continue;
                    }
                    map.put("work_code", workCode);
                } catch (Exception e) {
                    historyDetailPO.setOperateDetail(Util.null2String(e.getMessage()));
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                }
                map.put("creator", user.getUID());
                map.put("delete_type", 0);
                map.put("create_time", new Date());

                String showOrder = Util.null2String(map.get("show_order"));
                if (StringUtils.isBlank(showOrder)) {
                    Long maxShowOrder = MapperProxyFactory.getProxy(ResourceMapper.class).getMaxShowOrder();
                    if (null == maxShowOrder) {
                        maxShowOrder = 0L;
                    }
                    map.put("show_order", maxShowOrder + 1);
                }
                Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.RESOURCE, OperateTypeEnum.ADD, map, false).sync();
                if (isThrowError(syncMap) && StringUtils.isNotBlank(Util.null2String(syncMap.get("id")))) {
                    String ecResourceId = Util.null2String(syncMap.get("id"));
                    RecordInfo recordInfo = getSystemDataMapper().getHrmObjectByID(HRM_RESOURCE, ecResourceId);
                    map.put("uuid", recordInfo.getUuid());
                    ExtendInfoParams infoParams = ExtendInfoParams.builder().tableName("JCL_ORG_HRMRESOURCE").params(map).build();
                    map.remove("id");
                    MapperProxyFactory.getProxy(ExtMapper.class).insertTable(infoParams);
                    // 更新组织架构图
                    new HrmResourceTriggerRunnable(infoParams.getId()).run();
                    historyDetailPO.setOperateDetail("添加成功");
                    historyDetailPO.setStatus("1");

                } else {
                    historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                    historyDetailPO.setStatus("0");
                }
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
            } else if ("update".equals(operateType)) {
                // 查询对应ID
                if (resourceId == null) {
                    historyDetailPO.setOperateDetail("未找到对应数据");
                    historyDetailPO.setStatus("0");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                    continue;
                }
                map.put("update_time", new Date());
                map.put("id", resourceId);
                Map<String, Object> syncMap = new OrganizationSyncEc(user, LogModuleNameEnum.RESOURCE, OperateTypeEnum.UPDATE, map, false).sync();
                if (isThrowError(syncMap)) {
                    map.remove("id");
                    MapperProxyFactory.getProxy(ExtMapper.class).updateTable(ExtendInfoParams.builder().id(resourceId).tableName("JCL_ORG_HRMRESOURCE").params(map).build());
                    // 更新组织架构图
                    new HrmResourceTriggerRunnable(resourceId).run();
                    historyDetailPO.setOperateDetail("更新成功");
                    historyDetailPO.setStatus("1");

                } else {
                    historyDetailPO.setOperateDetail(Util.null2String(syncMap.get("message")));
                    historyDetailPO.setStatus("0");
                }
                OrgImportUtil.saveImportDetailLog(historyDetailPO);
            }
        }
        return importHistoryId;
    }

    /**
     * 方案职等职级导入
     */
    private Long jobLevelImport(String operateType, String excelFile) {

        Long importHistoryId = OrgImportUtil.saveImportLog("joblevel", operateType, user);
        JclImportHistoryDetailPO historyDetailPO;

        ImageFileManager manager = new ImageFileManager();
        manager.getImageFileInfoById(Util.getIntValue(excelFile));
        XSSFWorkbook workbook;
        try {
            workbook = new XSSFWorkbook(manager.getInputStream());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        int allRow = 0;
        for (int s = 0; s < workbook.getNumberOfSheets(); s++) {
            // 当前sheet
            XSSFSheet sheetAt = workbook.getSheetAt(s);
            int lastRow = sheetAt.getLastRowNum();
            allRow = allRow + lastRow;
            String relatedName;
            switch (s) {
                case 0:
                    relatedName = "方案页";
                    break;
                case 1:
                    relatedName = "职等页";
                    break;
                default:
                    relatedName = "职级页";
                    OrganizationAssert.isTrue(allRow > 0, "导入数据为空");
                    break;
            }
            short lastCellNum = sheetAt.getRow(0).getLastCellNum();
            List<ExtendInfoPO> extendInfoPOS = new ArrayList<>();

            // 遍历每一行数据
            nextRow:
            for (int i = 0; i <= lastRow; i++) {
                historyDetailPO = new JclImportHistoryDetailPO();
                historyDetailPO.setPid(importHistoryId);
                XSSFRow row = sheetAt.getRow(i);
                if (null == row) {
                    continue;
                }
                // 组装待处理数据
                Map<String, Object> map = new HashMap<>();
                SchemePO schemePO = new SchemePO();
                SchemePO schemeNew;
                GradePO gradePO = new GradePO();
                GradePO gradeNew = new GradePO();
                LevelPO levelPO = new LevelPO();
                LevelPO levelNew;

                historyDetailPO.setRowNums(String.valueOf(i + 1));
                for (int cellIndex = 0; cellIndex < lastCellNum; cellIndex++) {
                    XSSFCell cell = row.getCell((short) cellIndex);
                    String cellValue = getCellValue(cell).trim();
                    if (i == 0) {
                        // 首行 初始化字段信息
                        ExtendInfoPO extendInfoPO = importFieldsMap.get(cellValue);
                        extendInfoPOS.add(extendInfoPO);
                    } else {
                        ExtendInfoPO infoPO = extendInfoPOS.get(cellIndex);
                        // 数据校验
                        if (infoPO.getIsrequired() == 1 && StringUtils.isBlank(cellValue)) {
                            historyDetailPO.setRelatedName(relatedName);
                            historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "为必填项");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }

                        Object reallyValue;
                        try {
                            // 数据格式转换
                            reallyValue = OrgImportUtil.getReallyValue(infoPO, cellValue);
                        } catch (Exception e) {
                            historyDetailPO.setRelatedName(relatedName);
                            historyDetailPO.setOperateDetail(cellValue + "转换失败");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }
                        if (StringUtils.isNotBlank(cellValue) && StringUtils.isBlank(Util.null2String(reallyValue)) && !"ec_company".equals(infoPO.getFieldName()) && !"ec_department".equals(infoPO.getFieldName())) {
                            historyDetailPO.setOperateDetail(infoPO.getFieldNameDesc() + "数据转换失败,未找到对应数据");
                            historyDetailPO.setStatus("0");
                            OrgImportUtil.saveImportDetailLog(historyDetailPO);
                            continue nextRow;
                        }
                        map.put(infoPO.getFieldName(), reallyValue);
                        // 方案页校验(是否重复)
                        if ("scheme_no".equals(infoPO.getFieldName()) && s == 0 && "add".equals(operateType)) {
                            List<SchemePO> schemePOS = MapperProxyFactory.getProxy(SchemeMapper.class).listByNo(Util.null2String(reallyValue));
                            if (schemePOS.size() > 0) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("方案编号[" + reallyValue + "]与原有编号重复");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                        }
                        // 职等页校验(方案是否存在)
                        if ("scheme_no".equals(infoPO.getFieldName()) && s == 1) {
                            List<SchemePO> schemePOS = MapperProxyFactory.getProxy(SchemeMapper.class).listByNo(Util.null2String(reallyValue));
                            if (schemePOS.size() == 0) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + reallyValue + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                        }
                        // 职级页校验(方案是否存在)
                        if ("scheme_no".equals(infoPO.getFieldName()) && s == 2) {
                            List<SchemePO> schemePOS = MapperProxyFactory.getProxy(SchemeMapper.class).listByNo(Util.null2String(reallyValue));
                            if (schemePOS.size() == 0) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + reallyValue + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue nextRow;
                            }
                        }
                        // 职级页校验(职等是否存在)
                        if ("level_no".equals(infoPO.getFieldName()) && s == 2) {
                            String[] split = Util.null2String(reallyValue).split(",");
                            if (split.length > 0) {
                                for (String value : split) {
                                    List<LevelPO> levelPOS = MapperProxyFactory.getProxy(LevelMapper.class).listByNo(value);
                                    if (levelPOS.size() == 0) {
                                        historyDetailPO.setRelatedName(relatedName);
                                        historyDetailPO.setOperateDetail("未找到编号为[" + value + "]的职等");
                                        historyDetailPO.setStatus("0");
                                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                        continue nextRow;
                                    }
                                }
                            }
                        }
                    }
                }
                // 校验、数据交互
                if (i == 0) {
                    continue;
                }

                map.put("creator", user.getUID());
                map.put("delete_type", 0);
                Date currDate = new Date();
                map.put("create_time", currDate);
                map.put("update_time", currDate);
                // 禁用表示为空，默认启用
                map.put("forbidden_tag", 0);
                map.put("is_used", 1);
                if ("add".equals(operateType)) {
                    switch (s) {
                        case 0:
                            schemePO.setSchemeNo((String) map.get("scheme_no"));
                            schemePO.setSchemeName((String) map.get("scheme_name"));
                            schemePO.setSchemeDescription((String) map.get("scheme_description"));
                            schemePO.setForbiddenTag((int) map.get("forbidden_tag"));
                            schemePO.setIsUsed(1);
                            schemePO.setCreator((long) user.getUID());
                            schemePO.setDeleteType((int) map.get("delete_type"));
                            schemePO.setCreateTime((Date) map.get("create_time"));
                            schemePO.setUpdateTime((Date) map.get("update_time"));
                            MapperProxyFactory.getProxy(SchemeMapper.class).insertIgnoreNull(schemePO);
                            break;
                        case 1:
                            String schemeNo = (String) map.get("scheme_no");
                            String levelNo = (String) map.get("level_no");
                            schemeNew = MapperProxyFactory.getProxy(SchemeMapper.class).getSchemeByNo(schemeNo);
                            if (schemeNew != null) {
                                levelNew = MapperProxyFactory.getProxy(LevelMapper.class).getLevelByNoAndSid(levelNo, schemeNew.getId());
                                if (levelNew != null) {
                                    historyDetailPO.setRelatedName("职等编号");
                                    historyDetailPO.setOperateDetail("职等页导入:同一方案" + schemeNo + "下，职等编号不可重复");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue;
                                } else {
                                    levelPO.setLevelNo(levelNo);
                                    levelPO.setLevelName((String) map.get("level_name"));
                                    levelPO.setDescription((String) map.get("level_description"));
                                    //关联方案id
                                    levelPO.setSchemeId(schemeNew.getId());
                                    levelPO.setForbiddenTag((int) map.get("forbidden_tag"));
                                    levelPO.setIsUsed(1);
                                    levelPO.setCreator((long) user.getUID());
                                    levelPO.setDeleteType((int) map.get("delete_type"));
                                    levelPO.setCreateTime((Date) map.get("create_time"));
                                    levelPO.setUpdateTime((Date) map.get("update_time"));
                                    MapperProxyFactory.getProxy(LevelMapper.class).insertIgnoreNull(levelPO);
                                }
                            } else {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + schemeNo + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }
                            break;
                        case 2:
                            String gradeNo2 = (String) map.get("grade_no");
                            String schemeNo2 = (String) map.get("scheme_no");
                            String levelNo2 = (String) map.get("level_no");

                            schemeNew = MapperProxyFactory.getProxy(SchemeMapper.class).getSchemeByNo(schemeNo2);
                            if (schemeNew == null) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + schemeNo2 + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }

                            gradePO.setGradeNo(gradeNo2);
                            gradePO.setGradeName((String) map.get("grade_name"));
                            gradePO.setDescription((String) map.get("grade_description"));
                            // 关联方案id
                            gradePO.setSchemeId(schemeNew.getId());
                            gradePO.setForbiddenTag((int) map.get("forbidden_tag"));
                            gradePO.setIsUsed(1);
                            gradePO.setCreator((long) user.getUID());
                            gradePO.setDeleteType((int) map.get("delete_type"));
                            gradePO.setCreateTime((Date) map.get("create_time"));
                            gradePO.setUpdateTime((Date) map.get("update_time"));
                            String[] split = levelNo2.split(",");
                            StringBuilder levelIds = null;
                            if (split.length > 0) {
                                for (String value : split) {
                                    levelNew = MapperProxyFactory.getProxy(LevelMapper.class).getLevelByNoAndSid(value, schemeNew.getId());
                                    if (levelNew == null) {
                                        historyDetailPO.setRelatedName(relatedName);
                                        historyDetailPO.setOperateDetail("未找到编号为[" + value + "]的职等");
                                        historyDetailPO.setStatus("0");
                                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                        continue nextRow;
                                    }
                                    if (levelIds != null) {
                                        levelIds.append(",").append(levelNew.getId());
                                    } else {
                                        levelIds = new StringBuilder(String.valueOf(levelNew.getId()));
                                    }
                                }
                            }
                            gradeNew = MapperProxyFactory.getProxy(GradeMapper.class).getGradeByNoAndSid(gradeNo2, schemeNew.getId());
                            if (gradeNew != null) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("同一方案[" + schemeNo2 + "]下，职级编号不可重复");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            } else {
                                // 关联职等id
                                gradePO.setLevelId(levelIds.toString());
                                MapperProxyFactory.getProxy(GradeMapper.class).insertIgnoreNull(gradePO);
                            }
                            break;
                        default:
                            break;
                    }
                    historyDetailPO.setOperateDetail("添加成功");
                    historyDetailPO.setStatus("1");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                } else if ("update".equals(operateType)) {
                    switch (s) {
                        case 0:
                            schemeNew = MapperProxyFactory.getProxy(SchemeMapper.class).getSchemeByNo((String) map.get("scheme_no"));
                            if (schemeNew != null) {
                                schemeNew.setSchemeName((String) map.get("scheme_name"));
                                schemeNew.setSchemeDescription((String) map.get("scheme_description"));
                                schemeNew.setUpdateTime((Date) map.get("update_time"));
                                MapperProxyFactory.getProxy(SchemeMapper.class).updateScheme(schemeNew);
                            } else {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + map.get("scheme_no") + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }
                            break;
                        case 1:
                            String schemeNo = (String) map.get("scheme_no");
                            String levelNo = (String) map.get("level_no");
                            schemeNew = MapperProxyFactory.getProxy(SchemeMapper.class).getSchemeByNo(schemeNo);
                            if (schemeNew != null) {
                                levelNew = MapperProxyFactory.getProxy(LevelMapper.class).getLevelByNoAndSid(levelNo, schemeNew.getId());
                                if (levelNew != null) {
                                    levelNew.setLevelName((String) map.get("level_name"));
                                    levelNew.setDescription((String) map.get("level_description"));
                                    //关联方案id
                                    levelNew.setSchemeId(schemeNew.getId());
                                    levelNew.setUpdateTime((Date) map.get("update_time"));
                                    MapperProxyFactory.getProxy(LevelMapper.class).updateLevel(levelNew);
                                } else {
                                    historyDetailPO.setRelatedName(relatedName);
                                    historyDetailPO.setOperateDetail("未找到编号为[" + schemeNo + "]的方案");
                                    historyDetailPO.setStatus("0");
                                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                    continue;
                                }
                            } else {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + schemeNo + "]的方案，无法更新");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }
                            break;
                        case 2:
                            String schemeNo2 = (String) map.get("scheme_no");
                            String levelNo2 = (String) map.get("level_no");
                            schemeNew = MapperProxyFactory.getProxy(SchemeMapper.class).getSchemeByNo(schemeNo2);
                            if (schemeNew == null) {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号为[" + schemeNo2 + "]的方案");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }
                            // 处理职级编号
                            String[] split = levelNo2.split(",");
                            StringBuilder levelIds = null;
                            if (split.length > 0) {
                                for (String value : split) {
                                    levelNew = MapperProxyFactory.getProxy(LevelMapper.class).getLevelByNoAndSid(value, schemeNew.getId());
                                    if (levelNew == null) {
                                        historyDetailPO.setRelatedName(relatedName);
                                        historyDetailPO.setOperateDetail("未找到编号为[" + value + "]的职等");
                                        historyDetailPO.setStatus("0");
                                        OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                        continue nextRow;
                                    }
                                    if (levelIds != null) {
                                        levelIds.append(",").append(levelNew.getId());
                                    } else {
                                        levelIds = new StringBuilder(String.valueOf(levelNew.getId()));
                                    }
                                }
                            }

                            if (gradeNew != null) {
                                gradeNew.setSchemeId(schemeNew.getId());
                                gradeNew.setGradeName((String) map.get("grade_name"));
                                gradeNew.setDescription((String) map.get("grade_description"));
                                // 关联职等id
                                gradeNew.setLevelId(levelIds.toString());
                                gradeNew.setUpdateTime((Date) map.get("update_time"));
                                MapperProxyFactory.getProxy(GradeMapper.class).updateGrade(gradeNew);
                            } else {
                                historyDetailPO.setRelatedName(relatedName);
                                historyDetailPO.setOperateDetail("未找到编号[" + schemeNo2 + "]的职级，无法更新");
                                historyDetailPO.setStatus("0");
                                OrgImportUtil.saveImportDetailLog(historyDetailPO);
                                continue;
                            }

                            break;
                        default:
                            break;
                    }
                    historyDetailPO.setOperateDetail("更新成功");
                    historyDetailPO.setStatus("1");
                    OrgImportUtil.saveImportDetailLog(historyDetailPO);
                }

            }
        }
        return importHistoryId;
    }

    /**
     * 读取表格文本内容
     */
    private String getCellValue(XSSFCell cell) {
        String cellValue = "";
        if (cell == null) {
            return "";
        }
        switch (cell.getCellType()) {
            case BOOLEAN: // 得到Boolean对象的方法
                cellValue = String.valueOf(cell.getBooleanCellValue());
                break;
            case NUMERIC:
                if (HSSFDateUtil.isCellDateFormatted(cell)) {// 先看是否是日期格式
                    SimpleDateFormat sft = new SimpleDateFormat("yyyy-MM-dd");
                    cellValue = sft.format(cell.getDateCellValue()); // 读取日期格式
                } else {
                    cellValue = String.valueOf(new Double(cell.getNumericCellValue())); // 读取数字
                    if (cellValue.endsWith(".0")) {
                        cellValue = cellValue.substring(0, cellValue.indexOf("."));
                    }
                }
                break;
            case FORMULA: // 读取公式
                cellValue = cell.getCellFormula();
                break;
            case STRING: // 读取String
                cellValue = cell.getStringCellValue();
                break;
        }
        cellValue = Util.toHtmlForHrm(cellValue);
        return cellValue;
    }


    /**
     * 校验重复编号
     */
    private boolean checkRepeatNo(String no, Long extendType, Long id) {
        if (StringUtils.isBlank(no)) {
            return true;
        }
        if (null != extendType) {
            switch (extendType.toString()) {
                case "1":
                    // return 0 == MapperProxyFactory.getProxy(CompMapper.class).checkRepeatNo(no, id);
                case "2":
                    return 0 == MapperProxyFactory.getProxy(DepartmentMapper.class).checkRepeatNo(no, id);
                case "3":
                    return 0 == MapperProxyFactory.getProxy(JobMapper.class).checkRepeatNo(no, id);
                default:
                    return false;
            }
        }
        return false;
    }

    /**
     * 生成导入模板文件
     */
    private String getTemplatePath(String importType, String columns) throws IOException {
        // 根据选择字段、生成对应的导入模板
        String[] split = columns.split(",");
        List<Object> columnList = new ArrayList<>(Arrays.asList(split));
        List<Object> schemeList = new ArrayList<>();
        List<Object> levelList = new ArrayList<>();
        List<Object> gradeList = new ArrayList<>();
        Object companyColumn = "";
        Object departmentColumn = "";
        // 排序
        /*
        第一列：所属分部
        第二列：所属部门\上级部门\部门
         */
        Iterator<Object> iterator = columnList.iterator();
        while (iterator.hasNext()) {
            Object column = iterator.next();
            if ("所属分部".equals(column.toString())) {
                companyColumn = column;
                iterator.remove();
            }

            if ("所属部门".equals(column) || "上级部门".equals(column)) {
                departmentColumn = column;
                iterator.remove();
            }
            if ("方案编号".equals(column)) {
                schemeList.add(column);
                levelList.add(column);
                gradeList.add(column);
            }
            if ("方案名称".equals(column)) {
                schemeList.add(column);
            }
            if ("方案说明".equals(column)) {
                schemeList.add(column);
            }
            if ("职等编号".equals(column)) {
                levelList.add(0, column);
                gradeList.add(column);
            }
            if ("职等名称".equals(column)) {
                levelList.add(1, column);
            }
            if ("职等说明".equals(column)) {
                levelList.add(column);
            }
            if ("职级编号".equals(column)) {
                gradeList.add(0, column);
            }
            if ("职级名称".equals(column)) {
                gradeList.add(1, column);
            }
            if ("职级说明".equals(column)) {
                gradeList.add(column);
            }
        }
        if (StringUtils.isNotBlank(departmentColumn.toString())) {
            columnList.add(0, departmentColumn);
        }

        if (StringUtils.isNotBlank(companyColumn.toString())) {
            columnList.add(0, companyColumn);
        }

        List<List<Object>> excelSheetData = new ArrayList<>();
        excelSheetData.add(columnList);
        String outPutPath = GCONST.getRootPath() + "hrm" + File.separator + "import" + File.separator + "template" + File.separator + user.getUID();
        XSSFWorkbook sheets = new XSSFWorkbook();
        if ("joblevel".equals(importType)) {
            excelSheetData.clear();
            excelSheetData.add(schemeList);
            ExcelUtil.genWorkbookV3(sheets, excelSheetData, 0, "方案");
            excelSheetData.clear();
            excelSheetData.add(levelList);
            ExcelUtil.genWorkbookV3(sheets, excelSheetData, 1, "职等");
            excelSheetData.clear();
            excelSheetData.add(gradeList);
            ExcelUtil.genWorkbookV3(sheets, excelSheetData, 2, "职级");
        } else {
            sheets = ExcelUtil.genWorkbookV2(excelSheetData, importType);
        }
        String excelPath = outPutPath + File.separator + importType + ".xls";
        File excelPathFile = new File(excelPath);
        if (!excelPathFile.exists()) {
            String substring = excelPath.substring(0, excelPath.lastIndexOf(File.separator));
            File file = new File(substring);
            if (file.mkdirs()) {
                boolean newFile = excelPathFile.createNewFile();
                if (!newFile) {
                    throw new IOException(excelPath + "文件创建失败");
                }
            }
        }
        FileOutputStream out = new FileOutputStream(excelPathFile);
        sheets.write(out);
        out.flush();
        excelPath = "/hrm/import/template/" + user.getUID() + "/" + importType + ".xls";
        return excelPath;
    }

    private boolean isThrowError(Map<String, Object> resultMap) {
        return "1".equals(Util.null2String(resultMap.get("status"))) || "1".equals(Util.null2String(resultMap.get("sign")));
    }

    /**
     * 判断表头是否正确
     *
     * @param historyDetailPO
     * @param cellValue
     * @param infoPOList
     * @return
     */
    private boolean isBreak(JclImportHistoryDetailPO historyDetailPO, String cellValue, List<ExtendInfoPO> infoPOList) {
        String operateDetail = "";
        if (CollectionUtils.isEmpty(infoPOList)) {
            operateDetail = "[" + cellValue + "]未找到对应字段";
        }
        if (infoPOList.size() > 1) {
            operateDetail = "[" + cellValue + "]在表中定义多个,请修改后重新导入";
        }
        if (StringUtils.isNotBlank(operateDetail)) {
            historyDetailPO.setOperateDetail(operateDetail);
            historyDetailPO.setStatus("0");
            OrgImportUtil.saveImportDetailLog(historyDetailPO);
            return true;
        }
        return false;
    }

    /**
     * 判断负责人是否有重名人员、若有重复人员、给出反馈信息。不处理该条数据
     *
     * @param historyDetailPO
     * @param cellValue
     * @return
     */
    private boolean hasSameName(JclImportHistoryDetailPO historyDetailPO, String cellValue) {
        String operateDetail = "";
        if (StringUtils.isBlank(cellValue)) {
            return false;
        }
        List<Long> resourceIds = MapperProxyFactory.getProxy(EmployeeMapper.class).getResourceIdsByName(cellValue);
        if (CollectionUtils.isEmpty(resourceIds)) {
            operateDetail = "[" + cellValue + "]未找到对应人员";
        }
        if (resourceIds.size() > 1) {
            operateDetail = "[" + cellValue + "]查询到多个人员,请在导入完成后自行执行该负责人";
        }
        if (StringUtils.isNotBlank(operateDetail)) {
            historyDetailPO.setOperateDetail(operateDetail);
            historyDetailPO.setStatus("0");
            OrgImportUtil.saveImportDetailLog(historyDetailPO);
            return true;
        }
        return false;
    }

    /**
     * 判断所选关键字段是否查出多条数据
     *
     * @param historyDetailPO
     * @param keyField
     * @param keyFieldValue
     * @return
     */
    private boolean hasSameKeyFieldValue(JclImportHistoryDetailPO historyDetailPO, String keyField, String keyFieldValue) {
        String operateDetail = "";
        if (StringUtils.isBlank(keyFieldValue)) {
            return false;
        }
        List<Long> resourceIds = MapperProxyFactory.getProxy(ResourceMapper.class).getIdByKeyField(keyField, keyFieldValue);
        if (CollectionUtils.isEmpty(resourceIds)) {
            operateDetail = "[" + keyFieldValue + "]未找到对应人员";
        }
        if (resourceIds.size() > 1) {
            operateDetail = "[" + keyFieldValue + "]查询到多个人员,请确认";
        }
        if (StringUtils.isNotBlank(operateDetail)) {
            historyDetailPO.setOperateDetail(operateDetail);
            historyDetailPO.setStatus("0");
            OrgImportUtil.saveImportDetailLog(historyDetailPO);
            return true;
        }
        return false;
    }
}
